@@ -43,7 +43,6 @@ class WindowPreview extends St.Button {
             },
             'notify::minimized', this._updateVisible.bind(this),
             this);
-        this._updateVisible();
 
         global.display.connectObject('notify::focus-window',
             this._onFocusChanged.bind(this), this);
@@ -55,6 +54,12 @@ class WindowPreview extends St.Button {
         return this._window;
     }
 
+    vfunc_map() {
+        super.vfunc_map();
+        // initialize visibility after being added to the stage
+        this._updateVisible();
+    }
+
     _onFocusChanged() {
         if (global.display.focus_window === this._window)
             this.add_style_class_name('active');
