@@ -1,18 +1,22 @@
+// SPDX-FileCopyrightText: 2011 Erick Pérez Castellanos <erick.red@gmail.com>
+// SPDX-FileCopyrightText: 2011 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2017 Florian Müllner <fmuellner@gnome.org>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 // -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
-/* exported init enable disable */
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GObject from 'gi://GObject';
+import Meta from 'gi://Meta';
+import St from 'gi://St';
 
-const { Clutter, Gio, GObject, Meta, St } = imports.gi;
+import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
-const PopupMenu = imports.ui.popupMenu;
-
-const Me = ExtensionUtils.getCurrentExtension();
-
-const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
-const _ = Gettext.gettext;
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
 
 const WORKSPACE_SCHEMA = 'org.gnome.desktop.wm.preferences';
 const WORKSPACE_KEY = 'workspace-names';
@@ -22,32 +26,32 @@ const TOOLTIP_ANIMATION_TIME = 150;
 
 const MAX_THUMBNAILS = 6;
 
-let WindowPreview = GObject.registerClass(
 class WindowPreview extends St.Button {
-    _init(window) {
-        super._init({
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(window) {
+        super({
             style_class: 'workspace-indicator-window-preview',
         });
 
         this._delegate = this;
-        DND.makeDraggable(this, { restoreOnSuccess: true });
+        DND.makeDraggable(this, {restoreOnSuccess: true});
 
         this._window = window;
 
-        this.connect('destroy', this._onDestroy.bind(this));
-
-        this._sizeChangedId = this._window.connect('size-changed',
-            () => this.queue_relayout());
-        this._positionChangedId = this._window.connect('position-changed',
-            () => {
+        this._window.connectObject(
+            'size-changed', () => this.queue_relayout(),
+            'position-changed', () => {
                 this._updateVisible();
                 this.queue_relayout();
-            });
-        this._minimizedChangedId = this._window.connect('notify::minimized',
-            this._updateVisible.bind(this));
+            },
+            'notify::minimized', this._updateVisible.bind(this),
+            this);
 
-        this._focusChangedId = global.display.connect('notify::focus-window',
-            this._onFocusChanged.bind(this));
+        global.display.connectObject('notify::focus-window',
+            this._onFocusChanged.bind(this), this);
         this._onFocusChanged();
     }
 
@@ -56,11 +60,10 @@ class WindowPreview extends St.Button {
         return this._window;
     }
 
-    _onDestroy() {
-        this._window.disconnect(this._sizeChangedId);
-        this._window.disconnect(this._positionChangedId);
-        this._window.disconnect(this._minimizedChangedId);
-        global.display.disconnect(this._focusChangedId);
+    vfunc_map() {
+        super.vfunc_map();
+        // initialize visibility after being added to the stage
+        setTimeout(() => this._updateVisible());
     }
 
     _onFocusChanged() {
@@ -77,10 +80,13 @@ class WindowPreview extends St.Button {
             this._window.window_type !== Meta.WindowType.DESKTOP &&
             this._window.showing_on_its_workspace();
     }
-});
+}
 
-let WorkspaceLayout = GObject.registerClass(
 class WorkspaceLayout extends Clutter.LayoutManager {
+    static {
+        GObject.registerClass(this);
+    }
+
     vfunc_get_preferred_width() {
         return [0, 0];
     }
@@ -107,12 +113,15 @@ class WorkspaceLayout extends Clutter.LayoutManager {
             child.allocate(childBox);
         }
     }
-});
+}
 
-let WorkspaceThumbnail = GObject.registerClass(
 class WorkspaceThumbnail extends St.Button {
-    _init(index) {
-        super._init({
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(index) {
+        super({
             style_class: 'workspace',
             child: new Clutter.Actor({
                 layout_manager: new WorkspaceLayout(),
@@ -137,16 +146,13 @@ class WorkspaceThumbnail extends St.Button {
         let workspaceManager = global.workspace_manager;
         this._workspace = workspaceManager.get_workspace_by_index(index);
 
-        this._windowAddedId = this._workspace.connect('window-added',
-            (ws, window) => {
-                this._addWindow(window);
-            });
-        this._windowRemovedId = this._workspace.connect('window-removed',
-            (ws, window) => {
-                this._removeWindow(window);
-            });
-        this._restackedId = global.display.connect('restacked',
-            this._onRestacked.bind(this));
+        this._workspace.connectObject(
+            'window-added', (ws, window) => this._addWindow(window),
+            'window-removed', (ws, window) => this._removeWindow(window),
+            this);
+
+        global.display.connectObject('restacked',
+            this._onRestacked.bind(this), this);
 
         this._workspace.list_windows().forEach(w => this._addWindow(w));
         this._onRestacked();
@@ -244,17 +250,16 @@ class WorkspaceThumbnail extends St.Button {
 
     _onDestroy() {
         this._tooltip.destroy();
-
-        this._workspace.disconnect(this._windowAddedId);
-        this._workspace.disconnect(this._windowRemovedId);
-        global.display.disconnect(this._restackedId);
     }
-});
+}
 
-let WorkspaceIndicator = GObject.registerClass(
 class WorkspaceIndicator extends PanelMenu.Button {
-    _init() {
-        super._init(0.0, _('Workspace Indicator'));
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super(0.5, _('Workspace Indicator'));
 
         let container = new St.Widget({
             layout_manager: new Clutter.BinLayout(),
@@ -286,14 +291,11 @@ class WorkspaceIndicator extends PanelMenu.Button {
         this._workspaceSection = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(this._workspaceSection);
 
-        this._workspaceManagerSignals = [
-            workspaceManager.connect_after('notify::n-workspaces',
-                this._nWorkspacesChanged.bind(this)),
-            workspaceManager.connect_after('workspace-switched',
-                this._onWorkspaceSwitched.bind(this)),
-            workspaceManager.connect('notify::layout-rows',
-                this._updateThumbnailVisibility.bind(this)),
-        ];
+        workspaceManager.connectObject(
+            'notify::n-workspaces', this._nWorkspacesChanged.bind(this), GObject.ConnectFlags.AFTER,
+            'workspace-switched', this._onWorkspaceSwitched.bind(this), GObject.ConnectFlags.AFTER,
+            'notify::layout-rows', this._updateThumbnailVisibility.bind(this),
+            this);
 
         this.connect('scroll-event', this._onScrollEvent.bind(this));
         this._thumbnailsBox.connect('scroll-event', this._onScrollEvent.bind(this));
@@ -301,28 +303,19 @@ class WorkspaceIndicator extends PanelMenu.Button {
         this._updateThumbnails();
         this._updateThumbnailVisibility();
 
-        this._settings = new Gio.Settings({ schema_id: WORKSPACE_SCHEMA });
-        this._settingsChangedId = this._settings.connect(
-            `changed::${WORKSPACE_KEY}`,
-            this._updateMenuLabels.bind(this));
+        this._settings = new Gio.Settings({schema_id: WORKSPACE_SCHEMA});
+        this._settings.connectObject(`changed::${WORKSPACE_KEY}`,
+            this._updateMenuLabels.bind(this), this);
     }
 
     _onDestroy() {
-        for (let i = 0; i < this._workspaceManagerSignals.length; i++)
-            global.workspace_manager.disconnect(this._workspaceManagerSignals[i]);
-
-        if (this._settingsChangedId) {
-            this._settings.disconnect(this._settingsChangedId);
-            this._settingsChangedId = 0;
-        }
-
         Main.panel.set_offscreen_redirect(Clutter.OffscreenRedirect.ALWAYS);
 
         super._onDestroy();
     }
 
     _updateThumbnailVisibility() {
-        const { workspaceManager } = global;
+        const {workspaceManager} = global;
         const vertical = workspaceManager.layout_rows === -1;
         const useMenu =
             vertical || workspaceManager.n_workspaces > MAX_THUMBNAILS;
@@ -443,19 +436,16 @@ class WorkspaceIndicator extends PanelMenu.Button {
         let newIndex = global.workspace_manager.get_active_workspace_index() + diff;
         this._activate(newIndex);
     }
-});
-
-function init() {
-    ExtensionUtils.initTranslations();
 }
 
-let _indicator;
+export default class WorkspaceIndicatorExtension extends Extension {
+    enable() {
+        this._indicator = new WorkspaceIndicator();
+        Main.panel.addToStatusArea('workspace-indicator', this._indicator);
+    }
 
-function enable() {
-    _indicator = new WorkspaceIndicator();
-    Main.panel.addToStatusArea('workspace-indicator', _indicator);
-}
-
-function disable() {
-    _indicator.destroy();
+    disable() {
+        this._indicator.destroy();
+        delete this._indicator;
+    }
 }
